#!/usr/bin/env python3

"""
|==================|
| User protection. |
|==================|

Verifi les entree utilisateur.
Les professeurs nous ont toujours appris:
Le pire ennemi en informatique, c'est l'utilisateur!
Et bien toute les verification de saisie de l'utilisateur se font ici
"""

import ipaddress
import os
import re
import socket

from ...tools import Printer, id_
from ...communication import constants

def dns_ip_verification(dns, version):
    """
    Verfifie que le dns 'dns' soit bien un nom de domaine valide et que en plus de ca, il pointe bien ici, et c'est pas fini:
    il faut par dessus le marche qu'il prene en compte le bon protocole ip
    retourne True si toutes ces conditions sont satisfaites, False sinon.
    """
    with Printer("DNS verification...") as p:
        if version != 6 and version != 4:
            p.show("La version de protocole ip doit etre 6 ou 4, pas %s." % version)
            return False
    if not re.fullmatch(r"(?!\-)(?:[a-zA-Z\d\-_]{0,62}[a-zA-Z\d]\.){1,126}(?!\d+)[a-zA-Z\d]{1,63}", dns):
        p.show("Le nom de domaine doit satisfaire l'expression suivante: "
            "^(?!\\-)(?:[a-zA-Z\\d\\-_]{0,62}[a-zA-Z\\d]\\.){1,126}(?!\\d+)[a-zA-Z\\d]{1,63}$")
        return False
    try:
        ip = socket.gethostbyname(dns)
    except socket.gaierror:
        p.show("Ce nom de domaine n'existe pas!")
        return False
    p.show("Le domaine '%s' est associé a l'adresse '%s'." % (dns, ip))
    id_().update()
    if version == 6:
        if ipaddress.ip_address(ip) == id_().ipv6:
            return True
        elif ipaddress.ip_address(ip) == id_().ipv4_wan:
            p.show("Le DNS est bon, mais il est associer a une ipv4, pas 6, entrez le dans la case juste en dessous.")
            return False
        p.show("Le DNS ne pointe pas le bon endroit, il devrait pointer sur '%s'." % id_().ipv6)
        return False
    else:
        if ipaddress.ip_address(ip) == id_().ipv4_wan:
            return True
        elif ipaddress.ip_address(ip) == id_().ipv6:
            p.show("Le DNS est bon, mais il est associer a une ipv6, pas 4, entrez le dans la case juste au dessus.")
            return False
        p.show("Le DNS ne pointe pas le bon endroit, il devrait pointer sur '%s'." % id_().ipv4_wan)
        return False

def email_verification(email):
    """
    Retourne True si 'email'
    est une adresse couriel correcte.
    """
    import unidecode
    with Printer("Email verification...") as p:
        if re.fullmatch(
                r"[a-zA-Z0-9._-]{1,254}@[a-zA-Z0-9._-]{2,}\.[a-z]{2,4}",
                unidecode.unidecode(email)):
            p.show("Ok")
            return True
        p.show("Bad")
        return False

def paths_verification(paths, revert=False):
    """
    Verifie que la variable 'paths'
    ait une forme correcte pour decrire un ensemble
    de repertoires ou fichiers avec des options.
    S'assure aussi que les parametres soient coherents.
    Retourne True si la variable est ok, False si il y a un quoique.
    :param revert: Retourne le message d'erreur plutot qu'un booleen.
    """
    with Printer("Paths descriptor verification...") as p:
        with Printer("Data format verification..."):
            if not isinstance(paths, dict):
                message = "'paths' must be a dictionary, it is a %s." % type(paths).__name__
                p.show(message)
                return message if revert else False
            if "paths" not in paths:
                message = "Missing field 'paths'."
                p.show(message)
                return message if revert else False
            if "excluded_paths" not in paths:
                message = "Missing field 'excluded_paths'."
                p.show(message)
                return message if revert else False
            for val in paths.values():
                if not isinstance(val, list):
                    message = "Fileds have to be list, not %s." % type(val).__name__
                    p.show(message)
                    return message if revert else False
        with Printer("Data consistency check..."):
            # verification de la taille des champs
            for champs, val in paths.items():
                if champs not in ("paths", "excluded_paths"):
                    if len(val) != len(paths["paths"]):
                        message = "Il doit y avoir %d options a chaque fois. Or le champs %s en comporte %d." \
                            % (len(paths["paths"]), champs, len(val))
                        p.show(message)
                        return message if revert else False
            # verification que les chemins pointent tous quelque part
            for path in paths["paths"] + paths["excluded_paths"]:
                if not os.path.exists(path):
                    message = "Path %s does not exist!" % repr(path)
                    p.show(message)
                    return message if revert else False
                if not os.path.isabs(path):
                    message = "%s is not absolute." % repr(path)
                    p.show(message)
                    return message if revert else False
            # verification que les "excluded_paths" soient bien contenus dans les paths
            for excluded_path in paths["excluded_paths"]:
                est_inclu = False
                for path in paths["paths"]:
                    ex = os.path.abspath(os.path.realpath(excluded_path)) # on s'affranchit des liens symboliques et tout
                    pa = os.path.abspath(os.path.realpath(path))
                    if ex[:len(pa)] == pa and len(ex) > len(pa):
                        est_inclu = True
                        break
                if not est_inclu:
                    message = "The path %s has no parent." % repr(excluded_path)
                    p.show(message)
                    return message if revert else False
            # verification qu'il n'y ai pas de doublon
            if len(set(paths["paths"])) != len(paths["paths"]):
                message = "'paths': There is a duplicate path."
                p.show(message)
                return message if revert else False
            if len(set(paths["excluded_paths"])) != len(paths["excluded_paths"]):
                message = "'excluded_paths': There is a duplicate path."
                p.show(message)
                return message if revert else False
        return True

def padlock_verification(padlock):
    """
    Retourne True si 'padlock' est correcte.
    Retourne False dans le cas contraire.
    """
    with Printer("Padlock verification...") as p:
        if not isinstance(padlock, dict):
            p.show("'padlock' doit etre un dictionaire, c'est un %s." % type(padlock).__name__)
            return False
        if "cipher" not in padlock:
            p.show("Il maque le champs 'cipher'.")
            return False
        if "paths" not in padlock:
            p.show("Il manque le champs 'paths'.")
            return False
        if "break_time" not in padlock:
            p.show("Il manque le champs 'break_time'.")
            return False
        if "notify_by_email" not in padlock:
            p.show("Il manque le champs 'notify_by_email'.")
            return False
        if type(padlock["cipher"]) is not bool:
            p.show("'padlock[\"cipher\"]' doit etre un booleen, c'est un %s." % type(padlock["cipher"]))
            return False
        if not paths_verification(padlock["paths"]):
            p.show("The paths description is not correct")
            return False
        if type(padlock["break_time"]) is not int:
            p.show("'padlock[\"break_time\"]' doit etre un entier, c'est un %s." % type(padlock["break_time"]))
            return False
        if padlock["break_time"] < 0:
            p.show("'padlock[\"break_time\"]' doit etre positif.")
            return False
        if type(padlock["notify_by_email"]) is not bool:
            p.show("'padlock[\"notify_by_email\"]' doit etre un booleen, c'est un %s." % type(padlock["notify_by_email"]))
            return False
        return True

def port_verification(port):
    """
    S'assure que le port d'ecoute specifie soit correcte
    retourne True si c'est la cas et False sinon.
    """
    with Printer("Port verification...") as p:
        if type(port) is str:
            if not port.isdigit():
                p.show("'port' doit etre un entier positif.")
                return False
            port = int(port)
        if type(port) is not int:
            p.show("'port' doit etre un entier ou une chaine de caractere, pas un %s." % type(port))
            return False
        if port < 1 or port > 49151:
            p.show("'port' doit etre compris entre 1 et 49151.")
            return False
        try:
            sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            if id_().ipv4_lan:
                sock.bind((str(id_().ipv4_lan), port))
            else:
                sock.bind(("127.0.0.1", port))
            sock.listen()
            sock.close()
            sock = socket.socket(socket.AF_INET6, socket.SOCK_STREAM)
            if id_().ipv6:
                sock.bind((str(id_().ipv6), port))
            else:
                sock.bind(("::1", port))
            sock.listen()
            sock.close()
        except PermissionError:
            p.show("Le port %d nessecite d'avoir les droits d'administrateur, embetant!" % port)
            return False
        except OSError:
            p.show("Le port est deja utilise par une autre application.")
            return False
        if port in constants.RESERVED_PORTS:
            proche = sorted((set(range(1024, 49152, 1)) - set(constants.RESERVED_PORTS)), key=lambda p : abs(p-port))[0]
            p.show("Ce port %d est reservé par l'institut de l'IANA. Le port libre le plus proche est le port %d." % (port, proche))
        return True

def security_verification(security):
    """
    Fait des verifications afin de s'assurer
    que les champs presents dans 'security' soient valides et
    coherent entre eux. Retourne True si ils sont coherent et False si ils ne le sont pas.
    """
    with Printer("Security verification...") as p:
        if not isinstance(security, dict):
            p.show("'security' doit etre un dictionaire, c'est un {}.".format(
                type(security).__name__))
            return False
        
        # presence des bons champs
        champs = ["public_key", "private_key", "psw", "hash", "sel", "sentence_memory"]
        manque = set(champs) - set(security.keys())
        if manque:
            p.show("Missing %s keys." % ", ".join(map(repr, security.keys())))
            return False
        surplus = set(security.keys()) - set(champs)
        if surplus:
            p.show("To much keys (%s)." % ", ".join(map(repr, security.keys())))
            return False

        # bon types
        if not isinstance(security["public_key"], bytes):
            p.show("'public_key' must be bytes, no '%s'." % type(security["public_key"]).__name__)
            return False
        if not isinstance(security["private_key"], bytes):
            p.show("'private_key' must be bytes, no '%s'." % type(security["private_key"]).__name__)
            return False
        if security["psw"] != None and not isinstance(security["psw"], str):
            p.show("'psw' must be None or str, no '%s'."  % type(security["psw"]).__name__)
            return None
        if security["hash"] != None and not isinstance(security["hash"], str):
            p.show("'hash' must be None or str, no '%s'."  % type(security["hash"]).__name__)
            return None
        if not isinstance(security["sel"], bytes):
            p.show("'sel' must be bytes, no '%s'."  % type(security["sel"]).__name__)
            return None
        if not isinstance(security["sentence_memory"], str):
            p.show("'sentence_memory' must be str, no '%s'."  % type(security["sentence_memory"]).__name__)
            return None

        # bonne forme
        if not re.search(
                r"-----BEGIN PUBLIC KEY-----(.|\n)+-----END PUBLIC KEY-----",
                security["public_key"].decode(errors="ignore")):
            p.show("La clef publique n'est pas au format 'PEM'.")
            return False
        if not re.search(
                r"-----BEGIN RSA PRIVATE KEY-----(.|\n)+-----END RSA PRIVATE KEY-----",
                security["private_key"].decode()):
            p.show("La clef privee n'est pas au format 'PEM'.")
            return False
        if security["psw"] == "":
            p.show("Le mot de passe ne doit pas etre une chaine vide.")
            return False

        return True

def username_verification(username):
    """
    Retourne True si le 'username' est un
    username valide, retourne False le cas echeant.
    """
    import unidecode
    with Printer("Username verification...") as p:
        if re.fullmatch(
                r"[a-zA-Z]{3,30}\d{,5}(?:[ _-][a-zA-Z0-9]{1,30}?){0,5}",
                unidecode.unidecode(username)):
            p.show("Ok")
            return True
        p.show("Bad")
        return False

def schedules_verification(schedules):
    """
    S'assure que 'schedules' est un dictionaire qui represente
    correctement les horaires.
    """
    with Printer("Schedules verification...") as p:
        if not isinstance(schedules, dict):
            p.show("'schedules' doit etre un dictionaire, pas un %s." \
                % type(schedules).__name__)
            return False
        for day in ("monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"):
            if not day in schedules:
                p.show("The '%s' key is missing." % day)
                return False
            if not isinstance(schedules[day], dict):
                p.show("Chaque jour doit etre represente par un dictionaire, pas par un %s." \
                    % type(schedules[day]).__name__)
                return False
            for date, limitation in schedules[day].items():
                if not re.fullmatch(r"\d{1,2}:\d{1,2}", date):
                    p.show("Le format de la date doit satisfaire l'expression reguliere:"
                        " \\d{1,2}:\\d{1,2}. %s ne la respecte pas." % date)
                    return False
                h, m = date.split(":")
                if int(h) >= 24:
                    p.show("On ne peut pas metre une heure plus tardive que 23h.")
                    return False
                if int(m) >= 60:
                    p.show("Il n'y a pas plus de 60 minutes dans une heure!")
                    return False
                if not isinstance(limitation, int) and not isinstance(limitation, float):
                    p.show("Les valeurs doivent bool, int ou float, pas %s." \
                        % type(limitation).__name__)
                    return False
        return True

def temperature_verification(temperature):
    """
    S'assure que la temperature reflete bien une valeur
    de temperature coherente pour un processeur.
    """
    with Printer("Temperature verification...") as p:
        if not isinstance(temperature, int) and not isinstance(temperature, float):
            p.show("La temperature doit etre un nombre, pas par un %s." \
                % type(temperature))
            return False
        if temperature < 20 or temperature > 100:
            p.show("Le tempertaure doit etre comprise en tre 20°C et 100°C. "
                "%d n'est pas dans cette fourchette." % temperature)
            return False
        return True

def bandwidth_verification(downflow, rising_flow):
    """
    S'assure que les parametre soient corecte
    Renvoie True si les debit sont coherent, retourne False sinon.
    """
    with Printer("Bandwidth verification...") as p:
        if not isinstance(downflow, int) and not isinstance(downflow, float):
            p.show("'downflow' doit etre un nonmbre, pas un %s." \
                % type(downflow).__name__)
            return False
        if not isinstance(rising_flow, int) and not isinstance(rising_flow, float):
            p.show("'rising_flow' doit etre un nonmbre, pas un %s." \
                % type(rising_flow).__name__)
            return False
        if downflow < 0 or downflow > 125:
            p.show("'downflow' doit etre compris entre 0 et 125 Mio/s. "
                "%d ne fait pas parti de cet intervalle." % downflow)
            return False
        if rising_flow < 0 or rising_flow > 125:
            p.show("'rising_flow' doit etre compris entre 0 et 125 Mio/s. "
                "%d ne fait pas parti de cet intervalle." % rising_flow)
            return False
        return True

def recording_directory_verification(recording_directory):
    """
    S'assure que le repertoire 'recording_directory' existe bien
    et que l'on a les droit d'ecriture dedans.
    Retourne False si l'une des cvonditions n'est pas respecee, True sinon.
    """
    with Printer("Recording directory verification...") as p:
        if not isinstance(recording_directory, str):
            p.show("'recording_directory' doit etre une chaine de "
                "caractere, pas un %s." % type(recording_directory).__name__)
            return False
        if not os.path.isdir(recording_directory):
            p.show("\"%s\" n'est pas un repertoire existant." % recording_directory)
            return False
        try:
            with open(os.path.join(recording_directory, "tmp"), "w") as f:
                f.write("contenu test")
        except PermissionError:
            p.show("Il n'y a pas les droits d'ecriture dans "
                "le dossier \"%s\"." % recording_directory)
            return False
        else:
            os.remove(os.path.join(recording_directory, "tmp"))
        return True

def free_size_verification(free_size, path):
    """
    S'assure que 'free_size' soit correcte.
    """
    with Printer("Free size verification...") as p:
        if isinstance(free_size, str):
            if not free_size.isdigit():
                p.show("'free_size' doit etre un entier positif.")
                return False
            free_size = int(free_size)
        if not isinstance(free_size, int):
            p.show("'free_size' doit etre un entier ou une chaine "
                "de caractere, pas un %s." % type(free_size).__name__)
            return False
        try:
            import shutil
        except ImportError:
            return True
        else:
            dispo = int(shutil.disk_usage(path).total/2**20)
            if free_size > dispo:
                p.show("Le disque fait seulement %d Mio!" % dispo)
                return False
        return True

def listen_verification(listen):
    """
    S'assure que le nombre de connection simultane
    maximum admissible par le serveur soit correct.
    retourne True si c'est la cas et False sinon.
    """
    with Printer("Listen verification...") as p:
        if isinstance(listen, str):
            if not listen.isdigit():
                p.show("'listen' doit etre un entier positif.")
                return False
            listen = int(listen)
        if not isinstance(listen, int):
            p.show("'listen' doit etre un entier. Pas un %s." \
                % type(listen).__name__)
            return False
        if listen < 1:
            p.show("'listen' doit etre supperieur ou egal a 1.")
            return False
        return True

def network_name_verification(network_name):
    """
    Retourne True si 'network name' permet bien de
    se raccrocher a un cluster.
    """
    return True
    raise NotImplementedError("S'assurer que le nom de serveur est correcte.")

def port_forwarding_verification(port):
    """
    S'assure que la redirection de port annoncee soit bien coherente.
    Retourne True si c'est la cas et False si ce n'est pas le cas.
    """
    with Printer("Port forwarding verification...") as p:
        if isinstance(port, str):
            if not port:
                return True
            if not port.isdigit():
                p.show("'port' doit etre un entier positif.")
                return False
            port = int(port)
        if not isinstance(port, int):
            p.show("'port' doit etre un entier ou une chaine de "
                "caractere, pas un %s." % type(port).__name__)
            return False
        if port < 1 or port > 49151:
            p.show("'port' doit etre compris entre 1 et 49151.")
            return False

        return True
        raise NotImplementedError("Faire la verification que le port colle")

def access_token_verification(access_token):
    """
    Retourne True si l'access token Drobox est valide.
    """
    with Printer("Access token verification...") as p:
        if not isinstance(access_token, str):
            p.show("'access token' must be str, "
                "not %s." % type(access_token).__name__)
            return False
        import raisin.communication.dropbox as dropbox
        try:
            dropbox.Dropbox("id", access_token).connect()
        except KeyboardInterrupt as e:
            raise e from e
        except Exception as e:
            p.show(" and ".join(e.args))
            p.show("Imposible de se connecter avec cet acces token.")
            return False
        else:
            return True



#!/usr/bin/env python3

"""
Permet d'unifier les couleurs et la mise en
forme de tous les widgets tkinter.
"""

import os

from ...tools import Printer, temprep
from ..hmi import tkinter

JAUNE      = "#fee473"
POURPRE    = "#6a1433"
VERT_CLAIR = "#b8dc72"
VERT_FONCE = "#3f6317"

def theme(widget):
    """
    Initialise un panel de couleur qui fait
    pensser a du raisin.
    Decore 'widget' avec le bon theme
    retourne le widget.
    """
    if type(widget) is tkinter.Label:
        widget.configure(
            background=JAUNE,                   # couleur d'arriere plan
            foreground="#000",                  # couleur du texte
            )
    elif type(widget) is tkinter.Entry:
        widget.configure(
            background=VERT_CLAIR,              # couleur de fond
            foreground="#000",                  # couleur du texte
            selectbackground=VERT_FONCE,        # couleur de selection
            selectforeground=JAUNE,             # couleur du texte selectionne
            highlightcolor=VERT_FONCE,          # couleur du cadre qui indique qu'on a le focus
            highlightbackground=POURPRE,        # couleur du cadre quand il n'y a pas le focus
            highlightthickness=2,               # epaisseur de la bordure
            )
    elif type(widget) is tkinter.Button:
        widget.configure(
            background=POURPRE,                 # couleur de font en cas de non activite
            foreground=JAUNE,                   # couleur du texte en cas d'inactivite
            activebackground=VERT_FONCE,        # couleur de fond en cas d'activite
            activeforeground=JAUNE,             # couleur du texte en cas d'activite
            takefocus=0,                        # on rend les boutons inxessible par la touche 'Tab'
            )
    elif type(widget) is tkinter.Checkbutton:
        widget.configure(
            background=JAUNE,                   # couleur de fond en temps normal
            activeforeground=JAUNE,             # couleur du texte quand il y a le curseur
            activebackground=VERT_FONCE,        # couleur de font quand il y a le curseur
            highlightbackground=POURPRE,        # couleur de la bordure en temps normal
            highlightcolor=VERT_FONCE,          # couleur de la bordure quand il y a le focus
            highlightthickness=2,               # eppaisseur de la bordure
            selectcolor=VERT_CLAIR,             # couleur de la petite case
            anchor="w",                         # pour placer le wigjet a gauche quand il a trop de place
        )
    elif type(widget) is tkinter.Canvas:
        widget.configure(
            borderwidth=0,                      # c'est la taille de la bordure, donc il n'y a pas de bordure
            background=JAUNE,                   # couleur de fond
            highlightbackground=JAUNE,          # couleur du contour en cas de non selection
            height=16,                          # hauteur du canvas en pxl
            width=16,                           # largeur du canvas en pxl
        )
    elif type(widget) is tkinter.Frame:
        widget.configure(
            background=JAUNE                    # couleur de fond
            )
    elif type(widget) is tkinter.Radiobutton:
        widget.configure(
            background=JAUNE,                   # couleur de fond
            activeforeground=JAUNE,             # couleur du texte lorsque la souris est au-dessus du widget
            activebackground=VERT_FONCE,        # couleur d’arriere plan quand la souris est au-dessus du widget
            highlightbackground=POURPRE,        # couleur de la bordure en temps normal
            highlightcolor=VERT_FONCE,          # couleur de la bordure quand il y a le focus
            highlightthickness=2,               # eppaisseur de la bordure
            selectcolor=VERT_CLAIR,             # couleur de la petite case
            anchor="w",                         # pour placer le wigjet a gauche quand il a trop de place
            )
    elif type(widget) is tkinter.Spinbox:
        widget.configure(
            background=VERT_CLAIR,              # couleur de fond
            foreground="#000",                  # couleur des chiffres
            highlightbackground=POURPRE,        # couleur de la bordure quand il n'y a pas le focus
            highlightcolor=VERT_FONCE,          # couleur de la ligne de mise en valeur du focus lorsque le widget l’obtient
            selectbackground=VERT_FONCE,        # couleur de font de la selection
            selectforeground=JAUNE,             # couleur de fond quand l'item est selectionne
            highlightthickness=2,               # taille de la bordure
            buttonbackground=POURPRE,           # couleur d'arriere plan utilise par les fleches
            )
    elif type(widget) is tkinter.PanedWindow:
        widget.configure(
            orient="horizontal",                # oriantation des panneau, l'un a cote des autre et non pas les un au dessu des autres
            background=POURPRE,                 # couleur de fond (donc le cadre)
            borderwidth=2,                      # taille des bordures
            )
    elif type(widget) is tkinter.ttk.Notebook:
        style = tkinter.ttk.Style()
        style.configure("TNotebook.Tab",
            background=POURPRE,                 # couleur d'arriere plan des onglets non selectionnes
            focuscolor=POURPRE,                 # couleur des petits pointilles de focus
            foreground=JAUNE,                   # couleur du text dans les onglets
            )
        style.map("TNotebook.Tab", background=
            [("selected", JAUNE),               # couleur de l'onglet selectionne
            ("active", VERT_FONCE)],            # couleur de l'onglet sous la souris
            foreground=[("selected", "#000")])  # couleur du texte de l'onglet selectionne
    else:
        raise TypeError("Il est impossible de decorer un %s." % type(widget))
    return widget

class Icons:
    """
    Genere des icons
    """
    def __init__(self):
        self.icon_help = None
        self.icon_error = None
        self.icon_info = None
        self.icon_ok = None
        self.icon_warning = None
        self.icon_refresh = None
        self.icon_trash = None

    def help(self):
        """
        Retourne l'icon 'help'.
        """
        if self.icon_help:
            return self.icon_help
        with Printer("Loading help icon..."):
            with open(os.path.join(temprep(), "help.png"), "wb") as f: # creation d'une image de taille 16*16
                f.write(
                    b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x10\x00\x00\x00\x10\x08\x06\x00\x00\x00\x1f\xf3\xffa\x00\x00\x02\xc3IDATx\xdatS\x03sdA'
                    b'\x18\xbc\xba?r\xb6m\xdb\xb6m+\xb6mol\xdb\xb6m\xdb\\o%o\xfbf^\xac\xafj<\xdd\x9f\x97\x00\x985\x96_\xd4^O\x86&\x19\x19dp\'F\xc6\xc4\xdd'
                    b'\xfa\xb9\xffg\x02\x97\x92!\xbb\xf9\xb6\xa1\xc0\xc0%\x85I\xc8\xa9C\xdf\x10\x9f\x1dto\xe0\x9a\xc2\x907!\xf9#C\xffN\xe2f\x82\xd3\x1f\xc9y'
                    b'\t\xdb\xba\x87\xe1\x13S\x8c7j\xfe8\xf4\xdc\x8a\x1dt\xef\x1b[\x82\xf6\x9ea<\x96\xf7\x12\xd2\xbf\x143\x93@\xf6\x85\x92\xaf\xb0\xab\x8f\x8b'
                    b'\x072\x9e8\xf7\xc1\x01\xa9\x85\x8d\x18\xe6\x89\xd0\xd9;\x02\xaf\xe8b\\\xf9\xe6\x8c\xd7j\x01\xe8\x1e\xe0\xe1\x89\x82\xb7\x88b(\x96\xf5y'
                    b'\xcb\x1d#A\x07\xf9x_\xc6\x03\xe4\x8c\xcf:\xc1`\x18)*\x1b{0\xc4\x15\x82JBn=\xf6?\xb7\xc1\'\xdd0t\x12E[\xee\xb0\xee\xac\xa7\x04\x9a&\x1ei'
                    b'Rj6\x05\xd3q\xe9\xab\x13\xabq\xe7#\x0b<\x90\xf3\x06\x95\xd11\x06g\xbf\xb8\xe0\xd4G\'De\xd6\xc2\xc2\'\x93\xa1XJ\x90\x91\x9c\xdf@\xfdd\xc1'
                    b'\xeb\xae\xebc\xcb]\x13\xec~l\x89\x83/m\xa1\xeb\x92\n*\xdd\xfd<\\\xfa\xe9\x81+\xbf<\xa1\xe5\x92\x86\xcc\xd2\x16\xac\xba\xac\x9bA\t\xb8\x03'
                    b'\xc3\x02\xec}l\x8e5\xd7\xf4\xc6\xc1O,q\xe8\x95\x1d\xbe\xe8G@2:\x06\xa9T\n5N\nn\xcb\xf9\xe3\xaeB\x00>\x1bFa\x88\xc4g\xc5%\x1d\xee$\x01\xf6?'
                    b'\xb5\xc0\xc6[F\xd8E\xcc\xa6\x9aO~pb\x03F\xc59\xa2\x18\x0f\x95\x83\xf1T=\x14\xcf5\xc3\xf1\xc7"\x81\x04X\x8c\x95\x94`\xd2\x85\x8fZA\xd8v\xdf'
                    b'\x94\x04\xca\x1a\xc7\xdfqp\xe3\xaf\x17\xb2\xcb\xdb\x91[\xd9\x81\x97Z\xe1x\xa5\x1d\x89w\xfa1\xf8h\x14\x0b\xfb\xd0\x12\xe4Wub\xf5U\xbd\x8c'
                    b'\xc9 2\x01\xf1e\xd8\xf5\xd8\x02\x87_\xdb\xe1\xccg\x17\xdc\x94\xf1\x81\xb1w6,\x02\xf2\xf1F7\n\x9f\x8c\xe3\xf0\xdd<\t\xbf\xacR\x90S\xd5\x05'
                    b'\xe7\xf0"\x86\xb8\xa09\x99F!M\xcd\x07\xed\x10V\xfb\x85onx\xa9\x11\x8aI\xf9f\x96\x80\x9f\x04(c\x9f\x01NT%\x06FD\xd8v\xcf\x98M\xe3d!\xc9'
                    b'\xbcR\xf5\x13\xf6\x0c\xf2\xf0\xd38\n\x97I\xb4\xdfh\x87aR~[\x8f\x83\x9dc\xaa0"\x90\xe0\x93N\xa8h\xe5e\x1d\x99y\xa5\xfcZ\xcd_\xd4C\x02\x17'
                    b'\x9b\xd3\x00}\x8f,\xfc0\x8d\x83\xacm\n\x9c\xa2*PPK\x8a\x8a\x04\xee\x87a\x84h\xed5\xfd\xc9R\x9e\xdfL[\xef\x18\x0b\xed\x83\xf2\xa4y$x#|1\xb8'
                    b'DcI]7<c\xca\x98\x1d\xf7M\x85$\xf7\xb3\x9bi\xb1v^uE7\x83\x98\xc9\xa5c\xcd5\xfd\x0c\x1a\xb0\xff\xc3\x96\x9d\x01B+\x9e\x9ch\xc3mb\x00\x00\x00'
                    b'\x00IEND\xaeB`\x82')
            self.icon_help = tkinter.PhotoImage(file=os.path.join(temprep(), "help.png"))
            os.remove(os.path.join(temprep(), "help.png"))
            return self.icon_help
            
    def error(self):
        """
        Retourne l'icon 'error'.
        """
        if self.icon_error:
            return self.icon_error
        with Printer("Loading error icon..."):
            with open(os.path.join(temprep(), "error.png"), "wb") as f:
                f.write(
                    b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x10\x00\x00\x00\x10\x08\x06\x00\x00\x00\x1f\xf3\xffa\x00\x00\x01\xf6IDATx\xda\x84\x92\x03'
                    b'\x8c\x9cq\x10\xc5\xe7\x1c\xd4nP\xdb\xb6\xed6\xae\xe3\xe4b\x14A\x11\xd7\xb6m#\xa8m\xdb>\xdb\xc6\xeb{\xf9\xb2\xde\xcdM\xf2\xdb\x1d\xbc\x99'
                    b'\xef/\x0bf\x8f\xcd\xea=4\x9bv\xcfl\xfe\x1dr\x83\xfe5\xe6\xac*{F\xd1S\xb3\x03\xaf\xcc\x8a\xbe7m\x8a\xb8!C\xf0\x8f|n\xd3\x06w\x99\xbb\xcc\xda'
                    b'\xc5P\x83\x9e\x9b\xb5{a\x16\x97\xd0\xb3\'\xca\xcf\x9f\x07\xe2\xe3}\xa8\xb8u\x0b\xbf\xc6\x8d\xc3EjNQ\x1b\xf0e5gM\x9e\x0c\xdc\xbd\x0b<y\x02<x'
                    b'\xe0\x83+\x97\x1e\x1b\x8b\xd3\xd4\x1e\xf1^\x89\x96\x9d\xd0\xa2\x05p\xe0\x00\xf0\xea\x15P\\\x0c\\\xbe\x0c\x1c:$\xe4+\xe7\xd4\x18\x7f\xed\xd3'
                    b'\x07\x87\xd8\xe3>0\xed\xb9|\xee\\`\xf9r 7\x172\xe4\xe7\x03\xdb\xb7;\xc8\x97\xb1V\xb9x1J&L\xd0*\x8avk\x15:\xedo11\x80\x06\x88\x85\x0b\x81\xac'
                    b',\xc8\x90\x93\xe3@\xabLJBa\xef\xde\xc85\x13x@v\xb0\xd7tUq5k\x02\x83\x06y\x981\x03HO\x87\xcc\xd5\x9c\xdf\xae\x9d\x1a\xdd\xbc![\xd9k\xb7\xf9'
                    b'\xf3/:\x1ah\xdc\xd8\xc3\xd0\xa1@jj\xc0\x80<37\xaf\xc9&\r\xd0#\xf9\xc2\xa02<\xdc\xa1}{\x80\r\xaeF\xe1\xf2K8\xa4\x98Zq\x97\xac\xd7\x16\xae\xf2'
                    b' \xb8\x8d\xa2\xca\xc8HTr+\xaef$\'\x03\x1d;:\xc8\x97\xb1&\x8d\xb4\x07\xd8\xb3\xd6u\x95\x97\xcc\x0e\xfc\xabQ\x03h\xd4\x08X\xba\xd4Y\xfe\xe0\xc1'
                    b'NL\xe4+\xa7\x9a\xe2\x0f\xd4\xaef\x8f\xb9\xec\x02\']\x0c\x0f\x8f\xcbj\xd5\n\xe8\xd2\x05\xe8\xd7O\xff>\xb8r\xa9\xd4l\xa0v\x95\xff\x93>i\xd6\xee'
                    b'tDD\xdc\x0f\xee\x13\xa3F\x05\xe5\x03k\x1b\xa8YI\xad\x05\xb3\xc3\x9cz\x80K;S\xbdz\xd1S\x8a\xbf\xf0\xc5}"\x0f\xe8\x1f`N\xcbv\xbe\\\x85\xed2\xab'
                    b'\xa5G\xb2\x85\xd7\xb4\x89\xe8\xb4\xff\xaf\x17\x87F\x00\xb1/\xeb\xae,9\xe7K\x00\x00\x00\x00IEND\xaeB`\x82')
            self.icon_error = tkinter.PhotoImage(file=os.path.join(temprep(), "error.png"))
            os.remove(os.path.join(temprep(), "error.png"))
            return self.icon_error
    
    def info(self):
        """
        Retourne l'icon 'info'.
        """
        if self.icon_info:
            return self.icon_info
        with Printer("Loading info icon..."):
            with open(os.path.join(temprep(), "info.png"), "wb") as f:
                f.write(
                    b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x10\x00\x00\x00\x10\x08\x06\x00\x00\x00\x1f\xf3\xffa\x00\x00\x03\x13IDATx\xdab@\x07\x80\xbe'
                    b'\xc8\x01H\xb23\x08\xc0\xaf\xcc\x14\xe3\x94\xcf6b[g\xdb\x1e\x9dom\xdb\xb6\xad\xd9Yc\xb4\xb6\x8d:\xdb\xfe\xf2\xe7\xec\xaej\xbb;$\xb1l\xcb\xea'
                    b'#\tQ2\xa7\x8c[{lR\xd9m\x9d\xca~\xfb\x0c\xd6\x1eM\x88\xf5\x8f\xd3,\x93\xde\x06\x03g\xce|\xbcT\x1e^f\x15\\I\xdb\xe0\x15\xae\xdf\xbc\xc7\xb5\x9b'
                    b'w\x11\xfc\x11\xd6t^@\xe5Q\xccRED\xd4k\xc9\'O\x9e\xfc`\x91<\xba>K;D\xc7\xe0\x05Z\xfa\xce\xd3=|\x89+\xd7\xef0x\xf2\n-\xbd\xe7i\x15\xb6\x8e\x81'
                    b'\x0b\x04d\xb4\xb3H\x16\xde\xfaR\x81\xa5\xca\xa8\xb2\x90\xecN\xd4\xfa\xa1G\x98\xab\x1b\xe4\xc2\x95\xdb\x00\xdc\xbdw_\xe8\x03\xcf|\x1a\xe30f!'
                    b'\xb5l9\x11]\xf6(\xd99X\xbdd\xa3U\xfe\x95\x90\xac\x0e\x822\xdb\t~\xc4;(\xa9?\xc9\xad\xdbb\xf4\xf6S\xf8\xa7\xb7>\xb2\x0b|\x14\x13\x9a\xdd\xc1'
                    b'_\xaa,R\xd5\xb5\x9fH\x7f\xa9\xe2\xbd\x94\xfe5\x98\x85\xd5a\x1eV\x8fyD\x03\'B\xeb9.\xd0<\xa2\x11\xd3p\xc1#\x1b\x1f\xdbC\xea\xd8\xea\xa4\xe5'
                    b'\xcb\xbd\xd9L\xde\x94\xca\xbcU\x1e*i\x95Y\x06\xdb\\\xf5lq\xd6\xb1\xd5\xc5\xc0fW\x03\x81\xb9=\x0c\x9c\xbe\xc6\xd9K\xb7\xb8p\xf5\x0e\x1b\x85'
                    b'\xef\xb7\xe3\xa5L\xdc\x9a\xc5\xd8\r\xe9\x8c\xdb\x90\xc6\xb8\x8d\x19,7\xcf\xee\x94\xbe\xd8\x13\xc7\xaf\xc7J\xf8E\x04\xfcr\xa2\x8c_M*\xf8\xf1'
                    b'D9~y}\x00\xdc\xbas\x8f\xa9\xbbs\x98\xb03\x9f\x89;\xd4L\xd8\x96\xcb\x84\xad\xd9\x8c\xdf\x9c\xc5\xbf\x16yH\xf3\xf6\xc43mO>\xd3\xf6\x150}\x7f!'
                    b'\xd3e\xc5L\x93\x97 \x0bm\x01\xe0\xa6(0y\xaf\x9aI\xfb\x8a\x99\xb4\xb7\x90I\xbb5L\xdc\xa9f\xbc(\xb6\xc8V\x83\xf4\xa7Ev\xcf\x84\x03\xc5L\xde_'
                    b'\xcc\x14Y)S\x14eLWU \x0bo{6\xc1LE!\xd3\x94\x15L\x96\x971\xf9@\x89(T\xc4\xf8\xfdE\xacs\xc8\x1d\x966[\xa5\xbb\x8eW\x8adE93\x0fiYp\\\xcfW&FN\xc4'
                    b'w?+\xf0\xed\xf12\xbe\x16\xb6\xf9\xc7\xf4\xcc8X)\n\x953AQ\xc4\x11\xf7<\x7fI\xab\xedx\xef7s\r\x9f\x8b\x80\xef-\x8c\xfcfS\xc3_vu\xd8\xa7>\xbe\xc1'
                    b'\x9d\xbb\xf7\xf9\xd7F\xc7\xdf\xf6u\xfcb]\xc37\xa6F\xe6\x89&\x8bm\x0bo\x0c\x9f?\xff\x99\xf4?\x98\xf8h\x0e-r\xd0\xb3\xc8\xb1\x9e\x95\xaeM\x98'
                    b'\xc4\xf5\xd0=zU|\xe1\xe6#\xd4\xb6\x9ee\x83G\x03+\x84o\xb1u9+m\xf2Y\xba\xd3\xfd;\xe9E\xb0\t-O\xda\xe2Q\x8d<\xac\x9b\xfd\xc1\x9d\xc8\x82ZQ\nT'
                    b'\x04\xb5p \xa0\x99\xbd\x01\xad\xe2.\xbd\xacw\xaad\xe9\t\xf5\xa3\xe4\xd7\xc0%\xb2\xf4/\x95g\x19.)\xfdxd\x8c\xe2\x93s\xf2\x11\xba\xa7\x8f\xe0'
                    b'\x9c\xdc\xc7\x11\x9f\nT\xf6\xc9"\xf9\x1d\xd0\xdb\xdb;.2M\xef`\xe9_R\xef\x11S\x85{t\x15\xf6!\xe5\xda\xe8L\x83\xc5\xa9\xde\xde\x0f_\x8d\xff'
                    b'\x0f6\xafdJ\xdb\x7f\xaa\xf2\x00\x00\x00\x00IEND\xaeB`\x82')
            self.icon_info = tkinter.PhotoImage(file=os.path.join(temprep(), "info.png"))
            os.remove(os.path.join(temprep(), "info.png"))
            return self.icon_info

    def ok(self):
        """
        Retourne l'icon 'ok'.
        """
        if self.icon_ok:
            return self.icon_ok
        with Printer("Loading ok icon..."):
            with open(os.path.join(temprep(), "ok.png"), "wb") as f:
                f.write(
                    b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x10\x00\x00\x00\x10\x08\x06\x00\x00\x00\x1f\xf3\xffa\x00\x00\x01\xaaIDATx\xdac\x18( \x03'
                    b'\xc4\x11@l@\xaaF&FFF?aI\xc1\xfbr\x9a\xb2_\x81\xfcXR43333%\x1b\xda\xe9\x7f\x08\xce\xf1\xff\xcf\xc9\xc3y\x0c(&J\xacfF&&Fo\x13\x07\xc3\x0f\x8d'
                    b'\x8b\xaa\xff\x03]\xf0\x01(\xe6\x0f2\x94X\x03\x0c\xd5\x0cT\x9eO\xdd\xdd\xf7\xdf\xd8\xd9\xe0?\x90\xbf\x06\x88\xf9\xd1\x15i\x01q\x00\x10K\xa2'
                    b'\x89\x0b\t\x89\x0bnoZR\xfd?\xa7\'\x15\xe8t\x8e\xb7@1O\x0c\xdb\x05D\xf87\x05\xa6\x04\xfc\x93\x94\x97\xbc\r\x0c(_\xa0\x10\x0b\xc8\xe9,l,'
                    b'\xd9\x81\x19>\x7f&\xee\xe9\xfc\xaf\xef\xa0\xf3\x8f\x91\x91a#\xc8Pl\xceLv\x8fp\xfd\xd1\xbc\xa4\xfe\xbf\xba\x91\xea\'f\x16\xe6T\xa0\x98\xb6'
                    b'\xb2\x9e\xe2\x8d\xde\xed-\xffS;\xe2\xfe\xf3\n\xf3\x80B>\x12b8&\xe0\xe2\xe2\xe5\xda\x9e\xd3\x91\xfe\xbffA\xf9\x7fUC\xe5/\xacl,G\xe2k#\xfe5'
                    b'\xaf\xaf\xfao\xe9o\xf2\x1f\x18\x90\xa7\x80\xea\xe4\xf1\x05\x96\xa9\x92\x8e\xc2\xbb\xda\xc5\xe5\xff\xf3&\xa5\xffw\x8e\xb1\xff\xdf\xb4\xbe'
                    b'\xf2\x7frG\xf4\x7f\x11Y\xa1\xdf@\xf9\x1a \xe6\xc0\x9bP\x80Now\x89v\xf8S\xbe \xff\x7f\xc5\xe2\xfc\xff\x05\xb3\xd3\xfe\xdbGY\xfegfe~\x00\x94'
                    b'\xd7\x07\x85\x0b\xa1(\x93\xe2\x17\xe1\xbd\x14Y\x15\xf4?sR\xc2\xff\xc8\xba\x80\xff\x92\xaab\x7f\x81\xe2s\x81\x98\x8f\xd8x\x0f\x91\xd1\x90'
                    b'\xfc\xe8W\xe8\xf6\xdf\xd8[\xf7?\x0b\x1b\xf3K\xa0\x98\x0b)\t\x87\x19\x18\x95U\xbc"<\x9f\xd8\xb8\xd9~\x02\xf9\xd3\x81\x98\x97\xd4\x8c\xc3\x0c'
                    b'\xf5\xb3\x1d\x10\x0b\xe2\xf2;\x00\xfa,x\x06\\\xe1z\xcb\x00\x00\x00\x00IEND\xaeB`\x82')
            self.icon_ok = tkinter.PhotoImage(file=os.path.join(temprep(), "ok.png"))
            os.remove(os.path.join(temprep(), "ok.png"))
            return self.icon_ok

    def warning(self):
        """
        Retourne l'icon 'warning'.
        """
        if self.icon_warning:
            return self.icon_warning
        with Printer("Loading warning icon..."):
            with open(os.path.join(temprep(), "warning.png"), "wb") as f:
                f.write(
                    b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x10\x00\x00\x00\x10\x08\x06\x00\x00\x00\x1f\xf3\xffa\x00\x00\x01\xf3IDATx\xda\x9c\xd1\x03'
                    b'\x90\x96Q\x00\x85\xe1\x93m\xdb\xb6m\xdb\xb6m\xd7(\xdb\xb6m\xdb\xb6\xedZ\xdb\xfa\xf1v\xb3\xb5g\xe6\xf9p\r\xfd)\x97j*\x81\xa4\x08\nOn\xd7U'
                    b"\xf7'\x9d\xc5\xc9j\x1a()\x92\xfe'\xb7\xea(\xb1\xe9\xecm?+\x1eu\x92\xdf\xe0\\\xca\xfc\xbf\xb3/\xf2\xdf_\x0c\xec\x0f0oNU\xd7zIQ\xf5/\xb9YG"
                    b'\xf9_\x0f\x8fd\xf1u\xde\xce\x95+G\xf0u\xde\xc6\xab\xe1\x91\xac\x0b\x8b\xab\xe2?\x9d\xc7\xddF:\x17v\xaf5\xa7N\xeda\xe1\xc2\x85\xec\xd9\xb3'
                    b'\x99\xb0{-\xb9R_\xd7$\xc5\xfc\xdb\xd2\x9b\xbb\xcc\x8b\x0f\xd6\xe3\\\xb8\xb0\x939s\xe6p\xfc\xf8&\xb0\x1e\xc6en\\vUP\x0fI\x11\xf5\xab\\\xad'
                    b'\xad\xe8\x0f\xdb\xeb\xad\xdd\xa37p\x827o\xf60q\xe2D\xee\xdf\xdf\x04\x1c\xc4\xee\xde\x91\xbbm\xe5R#\x95\x92\xfdn\xf6q>\xfbR\x82}-\xb0\t\x8be'
                    b'#\xa3F\r%(h\x15\xb0\x0c\xec3\xf0\xd9\x97\x98\xe3U5GR\xe4\x1f\xaf-\xfd\xf3\xc1\n&\xa4-0\xd9\x98jL\xe1\xec\xd9\xa1\xc0\x18c\x941\x00\x82\xab'
                    b"\xf2t\x90B'\x16TA}\x9b;\r\xb5)\xe8~r\xa0\x83\xd1\xe5\x03\x0f\x8f\xf6\xac\\\xd9\x10'\xa7f@c\xa3\xb6Q\x99\xa0\xfbq9_[\xfb$E\xff8{]\x95w\x98"
                    b"\x15\x01l\xb9\x80\x92F\x99\x0f\x96.-D\xb6l\xd9\x187.'\x90\xd7\xc8nd4\xed\x92\xe30K\xac-\xad\x86\x1f\x0e\xf4~+\xdd\xb2zG\x03\x92\x19)\x8dT"
                    b'\x1f\xb8\xb9\xa5`\xda\xb4\xa4<}\x9a\x08\x88o\xc41b\x1a\xd1\xb0zG\xe4Fs=\x94\x14[\xf7\xdb\xc8\xc1aNd\x1c\xe6\xc5\xc4a~l\x1c\x16\xc4\xf9\xf0'
                    b'6>\x96\xcd\x8dnD\xc3aN\x14\xde\x9avogG4\xc4\xadVr\x93\x94Z\xcd3(Q\xff\x1c*\xdd#\x9b\xca\xff\x8f\x1c\xf1\xf4n\x9a`\x17@\x01\x13\x99\x98\x01'
                    b'\x00\x97\x0ed$\xac}\xef\xa8\x00\x00\x00\x00IEND\xaeB`\x82')
            self.icon_warning = tkinter.PhotoImage(file=os.path.join(temprep(), "warning.png"))
            os.remove(os.path.join(temprep(), "warning.png"))
            return self.icon_warning

    def refresh(self):
        """
        Retourne l'icon 'refresh'.
        """
        if self.icon_refresh:
            return self.icon_refresh
        with Printer("Loading refresh icon..."):
            with open(os.path.join(temprep(), "refresh.png"), "wb") as f:
                f.write(
                    b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x10\x00\x00\x00\x10\x08\x06\x00\x00\x00\x1f\xf3\xffa\x00\x00\x02&IDATx\xda\x8d\x93C{da'
                    b'\x14\x84\x83\x7f1\xb6m3\xb6\xad/7\xb6\xed\xb6\xed\x1e\xdb\xb6mk\x9d\xd5x9\xd8UN\xac\xf65\xaa\xea\xd6\xfb\x1c/\x00n\xd7\xc4\x03\x11\r\xd1'
                    b'\xb6`_G\xf7<\x12\x88\xb4\x04\xbe\x0f\xd6l\xb7\xbb\x14(\xbb\xcbM.\xba\x91Y\x91{1\xf5J\xe6\xa9\x84\x1f)\x87\xa3\x10\xbb;\x14\xe1F\x7f\x14'
                    b'\x1c\xcdB\xf6\xfeTlh[\xa5v(Pz;{}\xc1\xb5\x8c\xb3\xbc\x07\x8d\xd0>\x97\xc0\xfeV\x8f\xdd\xef\x0c\xb0\xbcQC\xffB\x0e\xd53\x11D\x0f;\x10g\x88'
                    b'\xc0\xa2\xd2\xb9\xfcQ\x02%7\xb3&\xe7_N;\xab|&\xc0\xc1\x8f6\x98_\xab`}\xa3\x85\xe1\xa5\x02\xea\xe7b\xc8\x1e\xf3 x\xd8\x86\xf6{\x8d\x08S\x04'
                    b'bv\xee\x0c\xe9(\x01\x8a\\\xd1v\xa7\x16\xfb\xde\x9ba\x7f\xa3\x87\xe2\x89\x10\xb9G2\xb0M\xb8\x1e\x14\x19+\xeb\x96 R\x1d\x8c\x10\x99?fq\xd3t'
                    b'\xe3>!\xe3D\xdc\x15\xc9\x83\x8e>G\xe9C\x1e\xfcd\x9b\xbb\xb7\xf0\xd6*F>47o\xe6\xfb\x99l\xea\x1e\x87%\xc6\xef\x0b\xffF-\xff\xa7\x96\xff\xee'
                    b'\x10o\xfc\xbd\xa9c5o\xecC\xd33\xa74OI\x9d\xe8Kf\x8c\x902B\xcaBu;\x19\x991/j\x99\x05(\xb72\x8a\xcc(2\xa3\xc8\xac\xfaq\xbeo\xc5\xfd\x9c\xe6'
                    b'\xb1Bd\xc6#\xb3\xdf\x03f\xff\xc9\xec\x9bC\xee\x84t\x0f!}?\xf2\x1a!U\x90Y\xb7\xf2\xb1\x90\xa8\x88Qw\xa1\x1cdve\xdc\xcb\x84T\'x\xd2\x8c\xce'
                    b'\xfb\xf5\xa0\xc8\xa0\xc8\xa0\xc8(>\xc9A\xf6\x80\xd7\x87T\xf2\xb8\x0b\t\xa6(\x10\xd2\x8aQ/\x13R)\xefa#\x8e|\xd9\xdd\x87t\xcf{\xd3(\xa4]\xf7'
                    b'\x9b\xfa\x90r\x07\xd3\xb1\xa0x\xceYB:y\x94\x00!\xe5\xb7\xdc\xaa\xc6\x9ew\xc6>\xa4\xc6W*h\x9eK!{"@\xe7\xed&\x94\x9c\xcaE\xb46\x14\xf3\x0bf'
                    b'\x9d%\xa4\xeb\x1d\xce\x02EV\x97\x9d\xcdC\xc1q\xd6\x8b\x14\x84\x14k\x9bW`E\xcd\xa2\x1f\x14\xf9\n!\xad \xa4\x93]\x0eS\xef\xe0P\xcb\xef=\x184'
                    b'\xc7\x02\x84\xd4\x97\x906x"\xd0\x03\x84\xac\x88\x99\xa8\xec\xca\xe6\x00\x00\x00\x00IEND\xaeB`\x82')
            self.icon_refresh = tkinter.PhotoImage(file=os.path.join(temprep(), "refresh.png"))
            os.remove(os.path.join(temprep(), "refresh.png"))
            return self.icon_refresh

    def trash(self):
        """
        Retourne l'icon 'trash'.
        """
        if self.icon_trash:
            return self.icon_trash
        with Printer("Loading trash icon..."):
            with open(os.path.join(temprep(), "trash.png"), "wb") as f:
                f.write(
                    b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x10\x00\x00\x00\x10\x08\x03\x00\x00\x00(-\x0fS\x00\x00\x00\x9cPLTE\x00\x00\x00\x00\x00'
                    b'\x00\x00\x00\x00\xff\xff\xff\xff\xff\xff\x00\x00\x00\xff\xff\xff\x00\x00\x00\x01\x01\x01\x04\x04\x04\x07\x07\x07\x13\x13\x13\x17\x17\x17'
                    b'\x19\x19\x19   """333@@@LLLMMMUUU]]]```bbbfffuuu{{{\x7f\x7f\x7f\x80\x80\x80\x88\x88\x88\x95\x95\x95\x97\x97\x97\x9a\x9a\x9a\xa6\xa6\xa6'
                    b'\xaa\xaa\xaa\xb5\xb5\xb5\xbb\xbb\xbb\xbf\xbf\xbf\xcb\xcb\xcb\xcc\xcc\xcc\xd4\xd4\xd4\xdf\xdf\xdf\xe1\xe1\xe1\xe5\xe5\xe5\xea\xea\xea\xec'
                    b'\xec\xec\xf0\xf0\xf0\xf4\xf4\xf4\xf6\xf6\xf6\xfd\xfd\xfd\xfe\xfe\xfe\xff\xff\xff\xeb\xbb\n%\x00\x00\x00\x07tRNS\x00 @\xc1\xc2\xdf\xf0\xed'
                    b'\xfcW\xa6\x00\x00\x00zIDATx\xda\x95\xcf\x03\x12\xc4P\x14D\xd1\x17\xdbN\xc6\xb6z\xffk\x8b\xf5\xcbs\x8a}\x8bMD"\x06\x02\xb5\xf3\x17[N\xc3'
                    b'\x0c\xdeM\xc2a\xa3D\x87F\xa6,\x0e\x1f\x81\xa0\xcc]AX\xce\xbdD\x92\x1f\xe1\n\x9d]x\x01\x11\xaeJ\x8c\xe2\xec\xdfS\x14\xcaq\x081/q\xde\x9f'
                    b'\xe1h_\x99\xb0\xbf\xad_\xdby\xe8tA|\xaa\x86\xdb\xb1\x94S\x1d\x08\xb9\xaet\xb4\xe4+\xce\xdf\xd7\xea]\x01i\xd0\x1b1\xb6\x80a\xba\x00\x00\x00'
                    b'\x00IEND\xaeB`\x82')
            self.icon_trash = tkinter.PhotoImage(file=os.path.join(temprep(), "trash.png"))
            os.remove(os.path.join(temprep(), "trash.png"))
            return self.icon_trash

icons = Icons()



import setuptools

with open("README.rst", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="raisin",
    version="0.0.13",
    author="Robin RICHARD (robinechuca)",
    author_email="raisin@ecomail.fr",
    description="Simple parallel, distributed and cluster computing",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    url="https://framagit.org/robinechuca/raisin",
    packages=setuptools.find_packages(),
    install_requires=["pycryptodomex", "cloudpickle"], # ces paquets serons installes d'office
    extras_require={
        "calculation": ["sympy", "giacpy", "numpy"],
        "tools": ["psutil>=5.1", "regex", "cloudpickle"],
        "graphical": ["tkinter", "matplotlib"],
        "security": ["pycryptodomex"]
        },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Operating System :: OS Independent",
    ],
    keywords=["parallel", "distributed", "cluster computing"],
    python_requires=">=3.6",
)



#!/usr/bin/env python3
#-*- coding: utf-8 -*-

import raisin

__all__ = ["CsvFileReader"]


def _find_delimiter_quotechar(file_path, delimiter, quotechar):
    """
    retourne le separateur et le quotechar le plus logic pour separer les donnees 'datas'.
    'datas (STR)' est le contenu direct d'un fichier csv, sans mise en forme prealable
    cherche les separateurs parmis:
        -"\t"
        -","
        -";"
        -" "
    cherche les quotechar parmis:
        -"'"
        -'"'
    """
    raise NotImplementedError

class CsvFileReader:
    """
    permet de lire un fichier csv et retourne un dictionaire de ce dernier
    """
    def __init__(self, file, encoding, signature, **kwargs):
        self.file_path = file
        self.encoding = encoding
        self.signature = signature
        self.delimiter = kwargs.get("delimiter", None)
        self.quotechar = kwargs.get("quotechar", None)
        assert self.quotechar in (None, "'", '"'), "les quotechar ne peuvent etre que ' ou \", pas %s" % self.quotechar
        if (not self.delimiter) or (not self.quotechar):            # si le separateur n'est pas specifier ou que le separateur de texte ne l'est pas non plus
            self.delimiter = _find_delimiter_quotechar(self.file_path, self.delimiter, self.quotechar)# on va essayer de le choisir intelligement
        self.file = open(self.file_path, "r", encoding=encoding)    # pointeur vers le fichier en lecture

    def close(self):
        """
        ferme le fichier
        n'a aucun effet si le fichier est deja ferme
        """
        with raisin.Printer("closing file %s..." % self.file_path, signature=self.signature):
            return self.file.close()

    def __iter__(self):
        """
        retourne peu a peu chaque ligne sous forme de liste
        """
        while 1:                                                    # on ne s'arrette jamais de lire le csv
            ligne_str = self.file.readline()                        # tant qu'il reste des lignes a lire
            if ligne_str == "":                                     # c'est a dire tant que l'on est par arrive a la fin
                break                                               # si tel est le cas, on sort du generateur
            ligne_str = ligne_str[:-1]                              # on supprime le '\n' present a la fin de la ligne
            liste = []                                              # initialisation de la liste
            in_str = False                                          # devient True si l'on est en plein milieu d'une chaine de caractere
            mot = ""                                                # c'est le mot en cours d'analyse
            echappement = 0                                         # c'est le nombre de caractere d'echappement
            for car in ligne_str:                                   # pour chaque caracteres
                if car == self.delimiter and not in_str:            # si on arrive sur un separateur
                    try:                                            # on va tenter de transformer en objet
                        data = eval(mot)                            # python, la suite de caractere precedente
                    except:                                         # si ca echoue une premiere fois
                        try:                                        # on essai une deuxieme fois
                            data = eval(mot.replace(",", "."))      # en remplaceant les virgules par des points (pour la gestion des nombres)
                        except:                                     # si meme ca ca ne fonctionne pas
                            data = mot                              # alors, on garde l'objet en str, sans changer son typage
                    liste.append(data)                              # alors on passe au paquet suivant
                    mot = ""                                        # on reinitialise tout pour la suite
                    continue                                        # puis on passe directement au caractere suivant
                if car == self.quotechar and not in_str:            # si l'on entre dans une chaine de caractere
                    assert mot == "", "une chaine de caractere n'est pas precede d'un separateur"# on fait une petite verification
                    in_str = True                                   # si la verification est bonne, on sait que l'on entre dans une chaine
                elif car == "\\" and in_str:                        # si on est dans une chaine et qu'il y a un caractere d'echappement
                    echappement = 1-echapement                      # alors on change la parite de echapement
                elif car == self.quotechar and in_str and not echappement:# si on arrive a la fin d'une chaine
                    echappement = 0                                 # alors on remet les compteurs a 0 pour le caractere d'echappement
                    in_str = False                                  # puis on signal que l'on est plus dans une chaine
                mot += car
            yield liste

    def read(self):
        """
        retourne un dictionaire de ce csv
        """
        return [l for l in self]

    def __enter__(self):
        """
        permet d'utiliser le mot clef 'with'
        """
        return self

    def __exit__(self, *args):
        self.close()    


#!/usr/bin/env python3
#-*- coding: utf-8 -*-

"""
Associ un "nom de dommaine" raisin, a
un vrai nom de dommaine ou a une ip et un port.
"""
import ipaddress
import os
import socket

import raisin

def get(rais_domain, signature):
    """
    Si le nom de domain est un nom de domaine recence par raisin.
    Les informations permettant de se connecter a ce serveur sont retourne.
    Sinon retourne None.
    L'objet retourne ressemble a ca:
        {
        "ipv4_wan": "37.45.168.54",
        "ipv4_lan": "192.168.1.24",
        "ipv6": "2a01:cb15:829b:c600:4ca7:56b3:fd1e:1719",
        "dns_ipv4": "machin_chose.ddns.net",
        "dns_ipv6": None,
        "port": 16395,
        "port_forwarding": None,
        "last_check": 1602927982.9591131,
        }
    """
    assert type(rais_domain) is str, "Le nom de domaine doit etre une chaine de caractere. Pas %s." % type(rais_domain)
    with raisin.Printer("Recherche du domain %s..." % rais_domain, signature=signature) as p:
        filename = os.path.join(os.path.expanduser("~"), "dns.py")
        if not os.path.exists(filename): # si le fichier n'existe pas
            p.show("File not found.")
            return None
        with open(filename, "r", encoding="utf-8") as f:
            dico = eval(f.read()) # c'est vraiment moche cette ligne niveau complexite!
        if rais_domain not in dico:
            p.show("Domain not present in the database.")
            return None
        infos = dico[rais_domain]
        p.show("Trouve: %s" % repr(infos))
        return infos

def is_ipv4(addr):
    """
    retourne True si il s'agit d'une adresse ipv4
    """
    assert type(addr) is str, "Pour ce test, l'ip doit etre une chaine de caractere."
    try:
        ipaddress.IPv4Address(addr)
        return True
    except ipaddress.AddressValueError:
        return False

def is_ipv6(addr):
    """
    retourne True si il s'agit d'une adresse ipv6
    """
    assert type(addr) is str, "Pour ce test, l'ip doit etre une chaine de caractere."
    try:
        ipaddress.IPv6Address(addr)
        return True
    except ipaddress.AddressValueError:
        return False

def is_domain(domain, signature):
    """
    retourne l'ip du dommaine si il s'agit d'un nom de domaine
    associe a une adresse ip. Retourn False sinon.
    """
    assert type(domain) is str, "Pour ce test, le nom de dommaine doit etre une chaine de caractere."
    with raisin.Printer("Attribution d'un potentiel dns a une ip...", signature=signature):
        if not raisin.re.fullmatch(r"(?!\-)(?:[a-zA-Z\d\-]{0,62}[a-zA-Z\d]\.){1,126}(?!\d+)[a-zA-Z\d]{1,63}", domain):
            return False
        try:
            return socket.gethostbyname(domain)
        except socket.gaierror:
            return False







#!/usr/bin/env python3

"""
permet de generer des clefs et de chiffrer / dechiffrer des donnees
"""

from Cryptodome.PublicKey import RSA
from Cryptodome.Cipher import PKCS1_OAEP, AES
import hashlib
import itertools
import os
import re
import secrets
import time
import uuid

from .application.settings import settings
from .application.hmi.dialog import question_reponse
import raisin.serialization.encrypt as encrypt
import raisin.serialization.decrypt as decrypt
from .tools import Printer


def rsa_keys(*, psw=None):
    r"""
    |===============================|
    | Genere une paire de clef RSA. |
    |===============================|

    parametres
    ----------
    :param psw: Permet de chiffrer la clef privee
    :type psw: str, None
        str: Est utilise comme codage symetrique.
        None: Ne chiffre rien du tout

    sortie
    ------
    :return: La clef privee suivie de la clef publique.
    :rtype: (bytes, bytes)

    exemple
    -------
    :Example:
    >>> from raisin.security import rsa_keys
    >>> rsa_keys()
    (b'-----BEGIN RSA PRIVATE KEY-----\nMIISKAIBAAK
    CBAEAk7rtT6a4lnBnpq0lStYuxzf7EGkDBIkW4YQGlisAuL
        ...
    42CYcXHQDymViYDtBLEkYSy9oIKUtgreb9M6sTVm1DgJBaC
    gInWOzU=\n-----END RSA PRIVATE KEY-----',
    b'-----BEGIN PUBLIC KEY-----\nMIIEIjANBgkqhkiG9
    w0BAQEFAAOCBA8AMIIECgKCBAEAk7rtT6a4lnBnpq0lStYu
        ...
    Ov9NLuUY5dS90iunJkV85zq9rBrLJvQKloLYhotULt\ngQI
    DAQAB\n-----END PUBLIC KEY-----')
    >>>
    """
    assert psw is None or isinstance(psw, str), \
        "Le mot de passe doit etre soit None, soit une "\
        +"chaine de caractere, pas un %s." % type(psw).__name__
    assert psw != "", "'psw' ne doit pas etre une chaine vide."
    
    with Printer("Generation of RSA keys..."):
        private_key = RSA.generate(1<<13)
        if psw:
            private_key_serialisee = private_key.exportKey("PEM", passphrase=psw.encode("utf-8"))
        else:
            private_key_serialisee = private_key.exportKey("PEM")
        public_key = private_key.publickey()
        public_key_serialisee = public_key.exportKey("PEM")

        return private_key_serialisee, public_key_serialisee

def change_private_key(private_key_pem, old_psw, new_psw, *, signature=None):
    """
    |====================================================|
    | Change le mot de passe qui protege la clef privee. |
    |====================================================|

    parametres
    ----------
    :param private_key_pem: Clef privee serialisee au format PEM.
    :type private_key_pem: bytes
    :param old_psw: Mot de passe qui protege actuellement la clef privee.
    :type old_psw: str, None
        str: Utilise le mot de passe pour dechiffrer la clef.
        None: La clef n'est pas chifree.
    :param new_psw: Nouveau mot de passe qui va bientot proteger la clef privee.
    :type new_psw: str, None
        str: La nouvelle clef va etre chiffree.
        None: Ne chiffre rien du tout.
    :param signature: N'importe quel objet qui permet d'afficher
        dans la bonne colone du terminal.

    sortie
    ------
    :return: La clef privee reserialisee
    :rtype: bytes

    exemple
    -------
    :Example:
    >>> from raisin.security import rsa_keys, change_private_key
    >>> priv, pub = rsa_keys()
    >>> priv = change_private_key(priv, None, "mot de passe")
    >>>
    """
    assert type(private_key_pem) is bytes,\
        "La clef privee doit etre de type 'bytes', pas %s." % type(private_key_pem)
    assert re.search(r"-----BEGIN RSA PRIVATE KEY-----(.|\n)+-----END RSA PRIVATE KEY-----",
        private_key_pem.decode()),\
        "La clef privee n'est pas au format 'PEM'."
    assert old_psw is None or type(old_psw) is str,\
        "L'ancien mot de passe doit etre soit None, soit une chaine de caractere, pas un %s." % type(old_psw)
    assert new_psw is None or type(new_psw) is str,\
        "Le nouveau mot de passe doit etre soit None, soit une chaine de caractere, pas un %s." % type(new_psw)

    if old_psw == None and new_psw == None:
        return private_key_pem
    elif new_psw == None:
        sentence = "Decipher the privated key..."
    elif old_psw == None:
        sentence = "Cipher the private key..."
    else:
        sentence = "Change the private key's password..."
    with Printer(sentence, signature=signature):
        return RSA.importKey(
            private_key_pem,
            passphrase=old_psw.encode("utf-8") if old_psw else None
            ).exportKey(
                "PEM",
                passphrase=new_psw.encode("utf-8") if new_psw else None
                )

def get_private_key(*, force=False, signature=None):
    r"""
    |======================================|
    | Recupere la clef privee de raisin.   |
    | Si elle n'existe pas, elle est cree. |
    | Le clef retournee est dechiffree.    |
    |======================================|

    parametres
    ----------
    :param force: Si la clef est chiffrer, le mot de passe est
        dans tous les cas demande a l'utilisateur.
    :type force: bool
    :param signature: N'importe quel objet qui permet d'afficher
        dans la bonne colone du terminal.

    sortie
    ------
    :return: la clef privee serialisee au format PEM.
    :rtype: bytes

    exemple
    -------
    :Example:
    >>> from raisin.security import get_private_key
    >>> get_private_key()
    b'-----BEGIN RSA PRIVATE KEY-----\nMIISJwIBAAKCBAEA2ZNhutNMYZ1B
        ...
    XX4fIHm7SNLPest13niyDEGGEI/lg==\n-----END RSA PRIVATE KEY-----'
    >>>
    """
    assert type(force) is bool, "'force' doit etre un booleen, pas %s." % type(force)

    with Printer("Recuperation de la clef privee raisin...", signature=signature):
        settings = load_settings()
        psw = request_psw(force=force, check=False)
        try:
            key_pem = settings["account"]["security"]["private_key"]
        except KeyError: # si il n'y a pas de clef privee
            settings["account"] = settings.get("account", {})
            settings["account"]["security"] = settings["account"].get("security", {})
            if "public_key" in settings["account"]["security"]:
                raise RuntimeError("Il y a une clef publique mais pas privee! "
                                    "Vous avez touches a quelque chose!")
            key_pem, public_key = rsa_keys(psw=psw, signature=signature)
            settings["account"]["security"]["private_key"] = key_pem
            settings["account"]["security"]["public_key"] = public_key
            dump_settings(settings)
        return change_private_key(key_pem, psw, None)

def get_public_key(*, signature=None):
    r"""
    |======================================|
    | Recupere la clef publique de raisin. |
    | Si elle n'existe pas, elle est cree. |
    |======================================|

    parametres
    ----------
    :param signature: N'importe quel objet qui permet d'afficher
        dans la bonne colone du terminal.

    sortie
    ------
    :return: la clef publique serialisee au format PEM.
    :rtype: bytes

    exemple
    -------
    :Example:
    >>> from raisin.security import get_public_key
    >>> get_public_key()
    b'-----BEGIN PUBLIC KEY-----\nMIIEIjANBgkqhkiG9w0BAQEFAAOCBA8AMIIECgKC
    BAEA2ZNhutNMYZ1BdGSFRlE7\nL7pnOiXxrcoj6xdp2bnEEkfPUl61630KwqEW5w1jIka3
        ...
    jspNlQ/qs3Zq4ukkSRfcc3+TFoKHYJlHc/QORv62\nsezGe249budYyoYaOK+hBsfG4xWR4
    vkipC323ptMYDL4jYd3iQj33nOWIZUtv/S\nkQIDAQAB\n-----END PUBLIC KEY-----'
    >>>
    """
    with Printer("Recuperation de la clef publique raisin...", signature=signature):
        settings = load_settings()
        try:
            return settings["account"]["security"]["public_key"]
        except KeyError: # si il n'y a pas de clef publique
            settings["account"] = settings.get("account", {})
            settings["account"]["security"] = settings["account"].get("security", {})
            if "private_key" in settings["account"]["security"]:
                raise RuntimeError("Il y a une clef privee mais pas publique! "
                                    "Vous avez touches a quelque chose!")
            psw = request_psw(force=False, check=False)
            private_key, public_key = rsa_keys(psw=psw, signature=signature)
            settings["account"]["security"]["private_key"] = private_key
            settings["account"]["security"]["public_key"] = public_key
            dump_settings(settings)
            return public_key

def change_psw(*, existing_window=None):
    """
    Change le mot de passe de raisin.
    """
    with Printer("Change the raisin security parameters..."):
        import raisin.application.hmi.psw as psw
        old_psw = request_psw(force=True)
        new_psw, sentence_memory, clair = psw.get_new_psw(existing_window=existing_window)
        sel = secrets.token_bytes(32)
        settings["account"]["security"]["sentence_memory"] = sentence_memory
        settings["account"]["security"]["private_key"] = change_private_key(
            settings["account"]["security"]["private_key"],
            old_psw,
            new_psw)
        if clair:
            settings["account"]["security"]["psw"] = new_psw
        else:
            settings["account"]["security"]["psw"] = None
        settings["account"]["security"]["sel"] = sel
        settings["account"]["security"]["hash"] = hashlib.sha512(sel + new_psw.encode("utf-8")).hexdigest()
        settings.flush()
        if old_psw != new_psw:
            request_psw.time = 0 # Pour eviter d'utiliser le mauvais mot de passe

class PswRequester:
    """
    Demande le mot de passe a l'utilisateur.
    """
    def __init__(self):
        self.psw = None # Le dernier mot de passe entre.
        self.time = 0   # La derniere date ou ce mot de passe a ete saisi.

    def __call__(self, force, *, message="", check=True, existing_window=None, signature=None):
        """
        |=====================================|
        | Recupere le mot de passe de raisin. |
        | Demande a l'utilisateur si besoin.  |
        |=====================================|

        parametres
        ----------
        :param force: Permet d'eviter de constament soliciter l'utilisateur.
        :type force: bool
            True: Demande le mot de passe a l'utilisateur. Meme si il est connu.
            False: Evite dans la mesure du possible de soliciter l'utilisateur.
        :param message: C'est une indication qui precise pourquoi on demande le mot de passe.
        :type message: bool
        :param check: Permet d'etre tres vigilant.
        :type check: bool
            True: S'assure que les clefs privees et publiques vont bien ensembles.
            False: Se contente de comparer les hash des mot de passe.
        :param existing_window: Permet de mieu gerer la nouvelle fenetre tkinter de dialogue.
        :param signature: N'importe quel objet qui permet d'afficher
            dans la bonne colone du terminal.

        sortie
        ------
        :return: Le mot de passe en clair.
        :rtype: str, None
        """
        def validate(security, psw=None):
            """
            Retourne True si les parametres sont coherent, False sinon.
            """
            if not psw:
                psw = security["psw"]
            if b"ENCRYPTED" in security["private_key"]:
                if not psw:
                    return False
                if hashlib.sha512(
                        security["sel"] + psw.encode("utf-8")
                        ).hexdigest() != security["hash"]:
                    return False
            test = uuid.uuid4().bytes
            try:
                if decrypt.decipher_rsa(
                        encrypt.cipher_rsa(
                            test,
                            security["public_key"],
                            1),
                        change_private_key(
                            security["private_key"],
                            psw,
                            None)
                        ) != test:
                    return False
            except KeyboardInterrupt as e:
                raise e from e
            except decrypt.DecryptError:
                return False
            return True

        # verifications
        assert type(force) is bool
        assert type(message) is str
        assert type(check) is bool

        security = settings["account"]["security"]
        
        # Les cas ou l'on peu eviter de poser la question.
        if (security["psw"] and not force) or (b"ENCRYPTED" not in security["private_key"]):
            if check:
                if not validate(security):
                    raise ValueError("Il y a une incoherence dans les parametres du mot de passe.")
            return security["psw"]                      # on le retourne tel qu'il est

        # si il a deja ete saisie il a moins d'une heure
        if self.psw and time.time() - self.time < 3600:
            if check:
                if not validate(security, self.psw):
                    raise ValueError("Il y a un probleme dans les parametres du mot de passe.")
            self.time = time.time()
            return self.psw                     # on le renvoi sans poser trop de questions

        # cas ou il faut poser la question
        validatecommand = lambda psw : validate(security, psw)
        surplus = '"%s\n"' % message if message else ""
        indication = " (%s)" % security["sentence_memory"] if security["sentence_memory"] else ""
        psw = question_reponse(
            "%sPlease enter the 'raisin' password%s:" % (surplus, indication),
            default=None,
            validatecommand=validatecommand,
            existing_window=existing_window,
            show="*")
        self.psw = psw
        self.time = time.time()
        return psw

request_psw = PswRequester() # permet de garder en memoire le mot de passe



#!/usr/bin/env python3

import email.mime.base
import email.mime.multipart
import email.mime.text
import imaplib
import itertools
import os
import re
import smtplib
import socket
import time

import raisin
from ..tools import Printer


class Mail:
    """
    permet de gerer des envois et des reception de couriel
    """
    def __init__(self, address, psw):
        self.address = address  # c'est l'address couriel en question
        self.psw = psw          # c'est le mot de passe qui permet de se connecter a la boite mail
        self.name = "mailbox '%s'" % self.address
        assert re.fullmatch(r"[a-zA-Z0-9._-]+@[a-zA-Z0-9._-]{2,}\.[a-z]{2,4}", self.address), "L'address '%s' n'est pas une address valide." % self.address

    def get_free_space(self, signature=None):
        """
        recupere l'espace disponible en octet
        """
        with Printer("Get free space of '%s'..." % self.name, signature=signature) as p:# on annonce que l'on va faire une operation risque
            ingoing_serv = self.get_ingoing_server(self.address)
            port = 993
            p.show("Connection a '%s' sur le port %d..." % (ingoing_serv, port))
            with imaplib.IMAP4_SSL(host=ingoing_serv, port=port) as imapserver:
                p.show("Autentification a '%s' avec le mdp: '%s'..." % (self.address, self.psw))
                imapserver.login(user=self.address, password=self.psw)
                with Printer("Finding main directory..."):
                    verification, dossiers = imapserver.list()
                    if verification != "OK":
                        raise imaplib.IMAP4.error("Impossible de recuperer le nom des dossiers dans %s." % self.name)
                    dossier = raisin.re.fullmatch(r"\(.*\)\s+\".*\"\s+(?P<dossier>.*)", dossiers[0].decode()).group("dossier")
                    p.show("It is '%s'." % dossier)
                with Printer("Getting quota..."):
                    quota = imapserver.getquotaroot(dossier)
                    p.show(str(quota))
                raise NotImplementedError

    def get_ingoing_server(self, addr, signature=None):
        """
        retourne le nom du serveur imap entran
        liee a l'address mail 'addr'
        """
        domain = addr.split("@")[1].lower()
        dico = {}
        return dico.get(domain, "imap.%s" % domain)

    def get_outgoing_server(self, addr, signature=None):
        """
        retourne le nom du serveur smtp sortant
        liee a l'address mail 'addr'
        """
        domain = addr.split("@")[1].lower()
        dico = {"att.net": "outbound.att.net",
                "bluewin.ch": "smtpauths.bluewin.ch",
                "club-internet.fr": "mail.club-internet.fr",
                "hotmail.com": "smtp.live.com",
                "outlook.com": "smtp.live.com",
                "sympatico.ca": "smtphm.sympatico.ca",
                "verizon.net": "outgoing.verizon.net",
                "yahoo.com": "mail.yahoo.com",
                "mailo.com": "mail.mailo.com"}
        return dico.get(domain, "smtp.%s" % domain)

    def send(self, address, subject, message, attachment=None, signature=None):
        """
        envoi un mail a l'address 'address'
        ce couriel a pour sujet 'subject', pour contenu 'message'
        et possiblement pour piece jointe 'attachment'
        """
        def generator_attachment(attachment):
            """
            cede au fur a meusure le binaire de chaque piece jointe
            lier a son nom
            """
            if attachment:
                if type(attachment) is bytes:
                    yield "attachement", attachment
                elif type(attachment) is str:
                    if os.path.isfile(attachment):
                        with open(attachment, "rb") as f:
                            yield os.path.basename(attachment), f.read()
                elif type(attachment) in (tuple, list, set):
                    for att in set(attachment):
                        for name, data in generator_attachment(att):
                            yield name, data
                else:
                    raise TypeError("la piece jointe n'est pas prise en charge")

        assert raisin.re.fullmatch(r"[a-zA-Z0-9._-]+@[a-zA-Z0-9._-]{2,}\.[a-z]{2,4}", address), "l'address %s n'est pas une address valide" % address
        
        outgoing_serv = self.get_outgoing_server(self.address)                  # address du serveur sortant
        
        with Printer("sending a mail to '%s' from '%s'..." % (address, self.address), signature=signature) as p:
            
            # mise en forme du message
            msg = email.mime.multipart.MIMEMultipart()
            msg["From"] = self.address
            msg["To"] = address
            msg["Subject"] = subject
            msg.attach(email.mime.text.MIMEText(message.encode("utf-8"), "plain", "utf-8"))
            for name, data in generator_attachment(attachment):
                part = email.mime.base.MIMEBase("application", "octet-stream")
                part.set_payload(data)
                email.encoders.encode_base64(part)
                part.add_header("Content-Disposition", "piece; filename= %s" % name)
                msg.attach(part)

            # envoi du message
            for i, (port, SmtplibClass) in enumerate(zip((587, 465), (smtplib.SMTP, smtplib.SMTP_SSL))):
                p.show("connection a '%s' sur le port %d..." % (outgoing_serv, port))
                try:
                    with SmtplibClass(host=outgoing_serv, port=port, timeout=10) as smtpserver:# connexion au serveur sortant (en precisant son nom et son port)
                        # smtpserver.set_debuglevel(True)                       # permet d'afficher tout ce qui se passe en arriere plan
                        smtpserver.ehlo()                                       # on s'assure que la connection soit faite
                        if smtpserver.has_extn("STARTTLS"):                     # si on peu encripter la session
                            p.show("encriptage de la session via STARTTLS...")  # alors on le fait
                            smtpserver.starttls()                               # connection TLS
                            smtpserver.ehlo()                                   # on se reidentifi avec la connection cryptee
                        p.show("autentification a '%s' avec le mdp: '%s'..." % (self.address, self.psw))
                        smtpserver.login(self.address, self.psw)                # autentification
                        p.show("envoi du message...")
                        smtpserver.sendmail(self.address, address, msg.as_string().encode("utf-8"))
                        return None
                except smtplib.SMTPException as e:
                    if i == 0:
                        p.show("echec! deuxieme tentative:")
                    else:
                        raise e from e

    def __repr__(self):
        return self.name


def send(address, subject, message, attachment=None, signature=None):
    """
    tente par tout les moyen d'envoyer ce mail
    ne retourne rien tant que le mail n'est pas envoye
    """
    for server in itertools.cycle(raisin.communication.mail.servers):
        try:
            server.send(address, subject, message, attachment, signature)
            break
        except (smtplib.SMTPResponseException, smtplib.SMTPServerDisconnected, socket.timeout):
            continue
        except socket.gaierror:                                 # si il n'y a pas internet
            time.sleep(30)                                      # on fait une pause en attendant qu'il revienne


servers = [ Mail("raisin@ecomail.fr", "raisin"),
            Mail("raisin@mailo.com", "5uRYSIE670dZ"),           # vrai mot de passe: "raisin"
            Mail("raisin.raisin@aol.com", "rjqqktugnledijkv"),  # vrai mot de passe: "grapegrape"
            Mail("serveurpython.oz@gmail.com", "raisinraisin")]

